### **AI 전문가 양성 과정 (상세 일정 및 강의 가이드) - 2025년 한국 버전**

**시작일: 2025년 6월 23일 (월)**

---

#### **Part 1. AI 개발을 위한 파이썬 마스터리**

**강의 접근 방식:**
*   **동기 부여:** AI/ML 분야에서 왜 파이썬이 필수적인지, 잘 갖춰진 개발 환경이 연구 및 개발 효율을 얼마나 높여주는지 실제 사례를 들어 설명하며 학습 동기를 부여합니다.
*   **실습 위주:** 모든 설치 과정과 명령어는 강사가 직접 화면을 공유하며 단계별로 함께 진행합니다. 수강생이 동일한 환경을 구축하고 명령어 실행 결과를 직접 확인하도록 유도합니다.
*   **개념 시각화:** 가상환경, 클래스, 객체 등 추상적인 개념은 그림이나 비유(예: '붕어빵 틀과 붕어빵')를 적극적으로 활용하여 직관적으로 이해시킵니다.
*   **오류 해결:** 설치나 실습 중 발생하는 오류는 숨기지 말고, 오히려 좋은 학습 기회로 삼습니다. 오류 메시지를 함께 읽고 해결하는 과정을 보여주며 문제 해결 능력을 길러줍니다.

**1주차: 과정 소개 및 AI 개발 환경 구축**
*   **6/23 (월):** 1일차 - 과정 소개, AI/ML 개요 및 **인공지능의 역사**
*   **6/24 (화):** 2일차 - Python, VSCode, Git 설치 및 기본 설정
*   **6/25 (수):** 3일차 - 가상환경(venv)의 이해와 실습
*   **6/26 (목):** 4일차 - Jupyter Notebook/Lab 기본 사용법 및 단축키
*   **6/27 (금):** 5일차 - 개발 환경 최종 점검 및 Q&A

**2주차: 파이썬 핵심 문법**
*   **6/30 (월):** 6일차 - 변수, 데이터 타입 (숫자형, 문자열, 불리언)
*   **7/1 (화):** 7일차 - 연산자 (산술, 비교, 논리)
*   **7/2 (수):** 8일차 - 조건문 (if, elif, else)
*   **7/3 (목):** 9일차 - 반복문 (for, while), `range()` 함수
*   **7/4 (금):** 10일차 - 함수 (정의, 호출, 인자, 반환)

**3주차: 파이썬 컬렉션**
*   **7/7 (월):** 11일차 - 리스트(List)와 튜플(Tuple) 심화
*   **7/8 (화):** 12일차 - 딕셔너리(Dictionary)와 세트(Set) 심화
*   **7/9 (수):** 13일차 - 문자열 포매팅과 유용한 내장 함수
*   **7/10 (목):** 14일차 - `collections` 모듈 (Counter, defaultdict 등)
*   **7/11 (금):** 15일차 - 리스트/딕셔너리 컴프리헨션(Comprehension)

**4주차: 객체 지향 프로그래밍 (OOP)**
*   **7/14 (월):** 16일차 - OOP의 개념, 클래스(Class)와 객체(Object)
*   **7/15 (화):** 17일차 - 생성자(`__init__`)와 인스턴스 변수
*   **7/16 (수):** 18일차 - 클래스 메소드와 인스턴스 메소드
*   **7/17 (목):** 19일차 - 상속(Inheritance)과 다형성(Polymorphism)
*   **7/18 (금):** 20일차 - '붕어빵 비즈니스' 예제로 OOP 종합 실습

**5주차: AI 핵심 라이브러리 (NumPy & Pandas)**
*   **7/21 (월):** 21일차 - NumPy 소개 및 다차원 배열(ndarray) 생성/연산
*   **7/22 (화):** 22일차 - Pandas 소개, Series와 DataFrame
*   **7/23 (수):** 23일차 - Pandas 데이터 선택, 필터링, 정렬
*   **7/24 (목):** 24일차 - Pandas Groupby 및 시각화 기초
*   **7/25 (금):** 25일차 - AI/ML 프로젝트 전체 흐름 이해

---

#### **Part 2. 핵심 AI 모델 개발 및 서빙**

**강의 접근 방식:**
*   **Why & What:** 각 라이브러리나 모델을 왜 배워야 하는지, 실제 AI 개발의 어떤 문제를 해결하는지 명확히 설명합니다. (예: "Pandas는 엑셀 데이터를 파이썬으로 자유자재로 다루기 위해 배웁니다.")
*   **데이터 중심:** 실제와 유사한 간단한 데이터셋(예: 타이타닉, 붓꽃)으로 시작하여, 데이터 로딩, 정제, 탐색, 시각화, 모델링으로 이어지는 전체 흐름을 반복적으로 보여줍니다.
*   **결과 해석:** 모델을 훈련시키고 예측 결과를 얻는 것에서 그치지 않고, `정확도`, `정밀도/재현율`, `과적합` 등 모델 평가 지표의 의미를 해석하고, 성능을 개선하는 방법을 함께 고민합니다.
*   **최신 트렌드 연결:** 딥러닝, 특히 LangChain과 RAG를 다룰 때는, ChatGPT와 같은 실제 서비스가 어떤 원리로 동작하는지 연결하여 설명하며 학습자의 흥미를 최고조로 이끕니다.

**6주차: 머신러닝 기초**
*   **7/28 (월):** 26일차 - 회귀(Regression) 모델 실습
*   **7/29 (화):** 27일차 - 분류(Classification) 모델 실습
*   **7/30 (수):** 28일차 - 군집(Clustering) 모델 실습
*   **7/31 (목):** 29일차 - 모델 평가 및 하이퍼파라미터 튜닝
*   **8/1 (금):** 30일차 - 머신러닝 프로젝트 종합 실습

**7주차: 딥러닝과 LLM 기초**
*   **8/4 (월):** 31일차 - 딥러닝 개요 및 PyTorch 기본, DNN 구현
*   **8/5 (화):** 32일차 - CNN 기본 원리 및 이미지 분류 모델 구현
*   **8/6 (수):** 33일차 - Transformer 아키텍처와 LLM의 이해
*   **8/7 (목):** 34일차 - LangChain 프레임워크 소개 및 기본 체인(Chain) 구성
*   **8/8 (금):** 35일차 - RAG(Retrieval-Augmented Generation) 개념 및 필요성

**☀️ 여름 휴가 (8/9 ~ 8/17) - 8/15 광복절 포함**

**8주차: LLM 애플리케이션 개발 및 서빙 준비**
*   **8/18 (월):** 36일차 - LangChain을 이용한 RAG 파이프라인 구축 실습 (1/2)
*   **8/19 (화):** 37일차 - LangChain을 이용한 RAG 파이프라인 구축 실습 (2/2)
*   **8/20 (수):** 38일차 - API 개념 및 FastAPI 소개
*   **8/21 (목):** 39일차 - FastAPI 기본 라우팅 및 Pydantic 데이터 유효성 검사
*   **8/22 (금):** 40일차 - 훈련된 ML 모델을 FastAPI로 서빙하기

**9주차: 프로덕션 레벨 API 서버**
*   **8/25 (월):** 41일차 - 프로덕션용 프로젝트 구조 설계 및 APIRouter
*   **8/26 (화):** 42일차 - Swagger UI를 이용한 API 문서 자동 생성 및 테스트
*   **8/27 (수):** 43일차 - Docker 개요 및 핵심 개념
*   **8/28 (목):** 44일차 - FastAPI 애플리케이션을 위한 Dockerfile 작성
*   **8/29 (금):** 45일차 - Docker Compose를 이용한 서비스 빌드 및 실행

*(9월부터 금요일은 대학원 수업으로 제외됩니다.)*

---

#### **Part 3. 프로덕션 MLOps 및 AI 심화**

**강의 접근 방식:**
*   **서비스 관점:** "잘 만든 모델을 어떻게 다른 사람에게 '쓸모있게' 제공할 것인가?" 라는 질문에서 출발합니다. API의 개념을 '식당의 메뉴판과 주문 시스템'에 비유하여 쉽게 설명합니다.
*   **코드 분리:** 왜 코드를 기능별로 분리하고, 프로젝트 구조를 체계적으로 관리해야 하는지(유지보수, 협업) 강조합니다. 실제 프로덕션 레벨의 프로젝트 구조를 제시하고 따라 해보도록 합니다.
*   **실용적 도구:** MLOps, 모델 최적화, AI 윤리 등 각 분야에서 실제로 널리 쓰이는 오픈소스 도구(Kubeflow, Feast, Triton, Fairlearn 등)를 직접 다루며 경험을 쌓습니다.
*   **미래 비전 제시:** MLOps, AI 윤리, 최신 트렌드를 소개하며 단순한 코더가 아닌, 비즈니스 문제를 해결하고 사회에 기여하는 'AI 전문가'로서의 성장 경로와 비전을 제시합니다.

**10주차: MLOps 심화 (Part 11)**
*   **9/1 (월):** 46일차 - MLOps 개요 및 파이프라인 오케스트레이션 (Kubeflow)
*   **9/2 (화):** 47일차 - CI/CD for ML (GitHub Actions)
*   **9/3 (수):** 48일차 - 실시간 모델 모니터링 (Prometheus, Grafana, Evidently AI)
*   **9/4 (목):** 49일차 - Feature Store의 이해와 활용 (Feast)

**11주차: 대규모 모델 최적화 (Part 12)**
*   **9/8 (월):** 50일차 - 효율적 분산 학습 (PyTorch FSDP, DeepSpeed)
*   **9/9 (화):** 51일차 - 모델 경량화 기법 (양자화, 가지치기)
*   **9/10 (수):** 52일차 - 지식 증류 및 고성능 추론 서버 (NVIDIA Triton)
*   **9/11 (목):** 53일차 - (미니 프로젝트) 경량화된 언어 모델 서빙 성능 비교

**12주차: 생성형 AI 및 에이전트 (Part 13)**
*   **9/15 (월):** 54일차 - LLM 효율적 파인튜닝 (PEFT, LoRA) 및 RAG 심화
*   **9/16 (화):** 55일차 - 자율 AI 에이전트 설계 및 구축 (ReAct, LangGraph)
*   **9/17 (수):** 56일차 - LLMOps & AgentOps (LangSmith)
*   **9/18 (목):** 57일차 - (미니 프로젝트) 자율 리서치 에이전트 만들기

**13주차: AI 윤리 및 전문가 로드맵 (Part 14, 10)**
*   **9/22 (월):** 58일차 - 설명가능 AI (XAI: LIME, SHAP)
*   **9/23 (화):** 59일차 - AI 편향성 탐지 및 완화 (Fairlearn)
*   **9/24 (수):** 60일차 - AI 규제와 거버넌스 (EU AI Act, 모델 카드)
*   **9/25 (목):** 61일차 - AI 전문가 커리어 패스 및 추천 학습 자료 (Part 10)

---

#### **Part 4. 최종 캡스톤 프로젝트 (Part 15)**

**14주차: 캡스톤 프로젝트 기획 및 개발 착수**
*   **9/29 (월):** 62일차 - (캡스톤) 프로젝트 아이디어 발상 및 팀 구성
*   **9/30 (화):** 63일차 - (캡스톤) 데이터 수집 및 전처리 전략 수립
*   **10/1 (수):** 64일차 - (캡스톤) 모델 개발 계획 및 역할 분담
*   **10/2 (목):** 65일차 - (캡스톤) 개발 시작 및 중간 점검 (1)

**🍂 추석 연휴 (10/6 ~ 10/8)**

**15주차: 캡스톤 프로젝트 개발 심화**
*   **10/13 (월):** 66일차 - (캡스톤) API 서버 구축 및 Docker 배포
*   **10/14 (화):** 67일차 - (캡스톤) 중간 점검 및 코드 리뷰 (2)
*   **10/15 (수):** 68일차 - (캡스톤) 최종 테스트 및 발표 자료 준비
*   **10/16 (목):** 69일차 - **(캡스톤) 최종 프로젝트 발표 및 회고**

**16주차: 예비 주간**
*   **10/20 (월):** 70일차 - (예비일) 추가 실습 또는 Q&A
*   **10/21 (화):** 71일차 - (예비일)
*   **10/22 (수):** 72일차 - (예비일)
*   **10/23 (목):** 73일차 - 과정 최종 정리 