# 16주 완성! AI 전문가 양성 과정: 파이썬부터 프로덕션 배포까지

![Course Banner](https://example.com/course_banner.png)  <!-- 이 부분은 나중에 적절한 이미지로 교체하세요 -->

"아이디어는 있는데, 어떻게 만들어야 할지 막막하신가요?"
"AI, 이제는 소비하는 것을 넘어 직접 만들고 싶으신가요?"

본 과정은 파이썬 기초 문법부터 시작하여, 실제 동작하는 AI 서비스를 직접 기획하고, 개발하며, 세상에 배포하는 전 과정을 경험하는 **16주 완성 실전형 AI 엔지니어링 코스**입니다. 막연한 아이디어를 현실로 만드는 '만들기 능력자'로 거듭나세요.

---

## 🚀 이 과정을 통해 얻을 수 있는 것

- **AI 서비스 개발의 A to Z**: 데이터 분석, 머신러닝/딥러닝 모델링, API 서버 개발, Docker를 이용한 배포까지, AI 제품 개발의 전체 사이클을 직접 경험하고 이해합니다.
- **실무 중심의 프로젝트 경험**: 체계적인 학습 후, '나만의 개성이 담긴 **캡스톤 프로젝트**'를 진행하며 현업 수준의 문제 해결 능력과 포트폴리오를 확보합니다.
- **생산성을 극대화하는 최신 기술 스택**: `FastAPI`, `Docker`, `PyTorch`, `LangChain` 등 현업에서 가장 수요가 높은 기술 스택을 사용합니다. `Cursor`, `GitHub Copilot` 같은 AI 네이티브 도구를 활용하여 개발 효율을 극대화하는 방법을 체득합니다.
- **지속 가능한 성장을 위한 발판**: MLOps의 기본 개념을 이해하고, 최신 AI 기술 트렌드를 읽는 눈을 기르며, AI 전문가로서 지속적으로 성장할 수 있는 학습 로드맵을 얻게 됩니다.

---

## 📚 전체 커리큘럼 개요

| 파트 (Part) | 주차 (Week) | 주제 (Topic) | 핵심 학습 내용 | 관련 문서 |
|:---:|:---:|:---|:---|:---|
| **1** | 1-5주차 | **AI 개발을 위한 파이썬 마스터리** | 개발 환경 구축부터 파이썬 핵심 문법, OOP, NumPy/Pandas까지 AI 개발의 기초를 탄탄히 다집니다. | [Intro](./part_0_introduction.md), [History](./part_0.1_history_of_ai.md), [Env](./part_1_ai_development_environment.md), [Python](./part_2_python_core_syntax.md), [Collections](./part_3_python_collections.md), [OOP](./part_4_object_oriented_programming.md), [Libs](./part_5_ai_core_libraries.md) |
| **2** | 6-9주차 | **핵심 AI 모델 개발 및 서빙** | 머신러닝, 딥러닝(CNN, Transformer)의 기초를 배우고, LangChain으로 LLM 챗봇을 만든 뒤 FastAPI와 Docker로 서빙합니다. | [ML](./part_6_machine_learning.md), [DL](./part_7_deep_learning.md), [Transformer](./part_7.2_transformer_and_llm_principles.md), [LangChain](./part_7.3_llm_application_development_with_langchain.md), [API](./part_8_model_serving_with_fastapi.md), [Prod API](./part_9_production_ready_api.md) |
| **3** | 10-13주차 | **프로덕션 MLOps 및 AI 심화** | MLOps, 모델 최적화/경량화, 자율 AI 에이전트, AI 윤리 등 현업 전문가를 위한 심화 주제를 다룹니다. | [MLOps](./part_11_mlops.md), [Optimization](./part_12_model_optimization.md), [Generative AI](./part_13_generative_ai.md), [Ethics](./part_14_ai_ethics.md), [Roadmap](./part_10_expert_path.md) |
| **4** | 14-15주차 | **최종 캡스톤 프로젝트** | 나만의 AI 서비스를 기획, 개발, 배포하며 실전 경험을 쌓고 포트폴리오를 완성합니다. | [Capstone](./part_15_capstone_project.md) |

---

## 🛠️ 기술 스택 (Tech Stack)

- **언어**: Python
- **핵심 라이브러리**: NumPy, Pandas, Scikit-learn, PyTorch, LangChain
- **API 서버**: FastAPI
- **배포 및 운영**: Docker, Git
- **개발 환경**: Visual Studio Code, Cursor, GitHub Copilot

---

## 📖 학습 자료

- **[전체 강의 계획서 (Course Schedule)](./AI_Expert_Course_Schedule.md)**: 16주 과정의 상세한 주차별 학습 목표와 내용을 확인하세요.
- **[파트별 용어집 (Glossaries)](./glossaries/)**: 과정 중에 등장하는 핵심 용어를 파트별로 다시 확인할 수 있습니다.
  - [Part 0: 과정 소개](./glossaries/part_0_glossary.md)
  - [Part 1: AI 개발 환경](./glossaries/part_1_glossary.md)
  - [Part 2: Python 핵심 문법](./glossaries/part_2_glossary.md)
  - [Part 3: Python 컬렉션](./glossaries/part_3_glossary.md)
  - [Part 4: 객체 지향 프로그래밍](./glossaries/part_4_glossary.md)
  - [Part 5: AI 핵심 라이브러리](./glossaries/part_5_glossary.md)
  - [Part 6: 머신러닝](./glossaries/part_6_glossary.md)
  - [Part 7: 딥러닝](./glossaries/part_7_glossary.md)
  - [Part 7.3: LangChain & LLM 앱 개발](./glossaries/part_7.5_glossary.md)
  - [Part 8: FastAPI](./glossaries/part_8_glossary.md)
  - [Part 9: Production-Ready API](./glossaries/part_9_glossary.md)
  - [Part 10: AI 전문가 과정](./glossaries/part_10_glossary.md)
  - [Part 11: MLOps](./glossaries/part_11_glossary.md)
  - [Part 12: 모델 최적화](./glossaries/part_12_glossary.md)
  - [Part 13: 생성형 AI](./glossaries/part_13_glossary.md)
  - [Part 14: AI 윤리](./glossaries/part_14_glossary.md)
  - [Part 15: 캡스톤 프로젝트](./glossaries/part_15_glossary.md)
- **[실습 코드 모음 (Source Code)](./source_code/)**: 각 파트별 실습 코드를 직접 실행하고 수정해볼 수 있습니다.

> [!NOTE]
> 이 과정은 '금도'님의 AI 강의 자료를 기반으로, 수강생의 학습 경험 극대화를 위해 커리큘럼을 재구성하고 내용을 심화시킨 결과물입니다. 원작자의 훌륭한 비유와 설명에 깊이 감사합니다. 