# Part 1: AI 개발 환경 - 핵심 용어집

- **가상환경 (Virtual Environment)**
  - **정의**: 프로젝트별로 독립된 파이썬 실행 환경을 만들어주는 도구. 각 프로젝트가 필요로 하는 라이브러리 버전 간의 충돌을 방지하고, 프로젝트의 의존성을 깔끔하게 관리할 수 있도록 돕습니다. 파이썬에서는 `venv`, `conda` 등이 대표적인 가상환경 관리 도구입니다.
  - **비유**: 프로젝트마다 별도의 '격리된 작업실'을 만들어, 각 작업실에 필요한 도구(라이브러리)들만 설치하여 서로 영향을 주지 않도록 하는 것과 같습니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **패키지 (Package)**
  - **정의**: 특정 목적과 관련된 여러 파이썬 모듈들을 모아놓은 디렉토리 구조. 패키지를 통해 관련된 코드들을 체계적으로 관리하고 재사용할 수 있습니다. `pip`는 파이썬의 공식 패키지 관리자로, PyPI(Python Package Index)에 등록된 수많은 패키지들을 쉽게 설치하고 관리할 수 있게 해줍니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **pip**
  - **정의**: 파이썬의 공식 패키지 관리 시스템. PyPI(Python Package Index)에 호스팅된 소프트웨어 패키지를 설치하고 관리하는 데 사용됩니다. `pip install <package_name>` 명령어로 라이브러리를 쉽게 설치할 수 있습니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **의존성 관리 (Dependency Management)**
  - **정의**: 프로젝트가 올바르게 실행되기 위해 필요한 외부 라이브러리(패키지)와 그 버전을 관리하는 작업. 파이썬에서는 `requirements.txt` 파일을 통해 프로젝트의 모든 의존성을 명시하고, `pip install -r requirements.txt` 명령어로 한 번에 설치하여 어떤 환경에서든 동일한 개발 환경을 재현할 수 있도록 합니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **IDE (Integrated Development Environment, 통합 개발 환경)**
  - **정의**: 코딩, 디버깅, 컴파일, 배포 등 프로그램 개발에 관련된 모든 작업을 하나의 인터페이스에서 처리할 수 있도록 도와주는 소프트웨어. 코드 자동 완성, 구문 강조, 디버거 등의 기능을 통해 개발 생산성을 크게 향상시킵니다. (예: VS Code, PyCharm)
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **Jupyter Notebook (주피터 노트북)**
  - **정의**: 코드, 텍스트(마크다운), 시각화 결과 등을 하나의 문서에 담아 대화형으로 프로그래밍할 수 있는 오픈소스 웹 애플리케이션. 코드 셀 단위 실행이 가능하여 데이터 분석, 머신러닝 모델링, 실험 기록 등 탐색적 프로그래밍에 널리 사용됩니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **커널 (Kernel)**
  - **정의**: Jupyter Notebook이나 JupyterLab과 같은 대화형 컴퓨팅 환경에서, 사용자가 작성한 코드를 실제로 해석하고 실행하는 '계산 엔진'. 각 노트북은 특정 커널(예: 특정 가상환경의 파이썬)에 연결되어, 해당 환경의 라이브러리와 설정을 사용하여 코드를 실행합니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **버전 충돌 (Version Conflict)**
  - **정의**: 하나의 시스템에서 두 개 이상의 프로젝트가 동일한 라이브러리의 서로 다른 버전을 필요로 할 때 발생하는 문제. 가상환경은 각 프로젝트에 독립적인 라이브러리 설치 공간을 제공하여 이 문제를 근본적으로 해결합니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **Google Colab (구글 코랩)**
  - **정의**: 구글에서 제공하는 클라우드 기반의 무료 Jupyter 노트북 환경. 별도의 개발 환경 설정 없이 웹 브라우저만으로 파이썬 코드를 작성하고 실행할 수 있으며, 특히 무료로 GPU 가속을 사용할 수 있어 머신러닝 및 딥러닝 모델 학습에 매우 유용합니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **AI-Powered IDE (AI 기반 통합 개발 환경)**
  - **정의**: 기존 IDE에 코드 생성, 수정, 디버깅, 질문 답변 등의 기능을 수행하는 AI 모델이 결합된 차세대 개발 환경. 개발자의 생산성을 극대화하고 반복적인 작업을 자동화합니다. (예: Cursor, GitHub Copilot)
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **Cursor**
  - **정의**: VS Code를 기반으로 제작된 AI 네이티브 코드 에디터. VS Code의 모든 기능과 확장 프로그램을 그대로 사용하면서, 프로젝트 전체의 맥락을 이해하는 AI 채팅, 인라인 코드 생성 및 수정, 자동 디버깅 등 강력한 AI 기능을 제공합니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **Code Generation (코드 생성)**
  - **정의**: 개발자가 자연어(주석 등)로 원하는 기능이나 로직을 설명하면, AI가 그에 맞는 코드 스니펫이나 전체 함수/클래스를 자동으로 작성해주는 기능.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md)

- **Context-aware AI (맥락 인지 AI)**
  - **정의**: 단순히 주어진 질문이나 명령어만 보는 것을 넘어, 현재 작업 중인 프로젝트의 파일 구조, 열려 있는 탭, 터미널 로그, 심지어 참조된 문서까지 종합적으로 파악하여 사용자의 의도에 가장 적합한 답변이나 코드를 제공하는 AI 기술. Cursor의 핵심적인 강점 중 하나입니다.
  - **관련 파트**: [Part 1: AI 개발 환경 완벽 구축 가이드](../part_1_ai_development_environment.md) 